@@ -84,7 +84,7 @@ def _format_message(message, name, config=None, pos=None):
 
 def _suppressed_warning(warning, config, stacklevel=2):
     warning_class = type(warning)
-    config.setdefault("_warning_counts", dict()).setdefault(warning_class, 0)
+    config.setdefault("_warning_counts", {}).setdefault(warning_class, 0)
     config["_warning_counts"][warning_class] += 1
     message_count = config["_warning_counts"][warning_class]
     if message_count <= conf.max_warnings:
@@ -164,7 +164,7 @@ def warn_unknown_attrs(element, attrs, config, pos, good_attr=[], stacklevel=1):
 
 _warning_pat = re.compile(
     r":?(?P<nline>[0-9?]+):(?P<nchar>[0-9?]+): "
-    + r"((?P<warning>[WE]\d+): )?(?P<rest>.*)$"
+    r"((?P<warning>[WE]\d+): )?(?P<rest>.*)$"
 )
 
 
@@ -255,6 +255,12 @@ class VOTableSpecWarning(VOWarning, SyntaxWarning):
     """
 
 
+class ModelMappingSpecWarning(VOWarning, SyntaxWarning):
+    """
+    The input model mapping XML mapping block violates the spec.
+    """
+
+
 class UnimplementedWarning(VOWarning, SyntaxWarning):
     """
     A feature of the VOTABLE_ spec is not implemented.
@@ -637,7 +643,7 @@ class W19(VOTableSpecWarning):
 
     message_template = (
         "The fields defined in the VOTable do not match those in the "
-        + "embedded FITS file"
+        "embedded FITS file"
     )
 
 
@@ -654,12 +660,12 @@ class W20(VOTableSpecWarning):
 class W21(UnimplementedWarning):
     """
     Unknown issues may arise using ``astropy.io.votable`` with VOTable files
-    from a version other than 1.1, 1.2, 1.3, or 1.4.
+    from a version not in 1.1 through 1.5.
     """
 
     message_template = (
-        "astropy.io.votable is designed for VOTable version 1.1, 1.2, 1.3,"
-        " and 1.4, but this file is {}"
+        "astropy.io.votable is designed for VOTable versions 1.1 through 1.5,"
+        " but this file is {}"
     )
     default_args = ("x",)
 
@@ -883,7 +889,9 @@ class W36(VOTableSpecWarning):
 class W37(UnimplementedWarning):
     """
     The 3 datatypes defined in the VOTable specification and supported by
-    ``astropy.io.votable`` are ``TABLEDATA``, ``BINARY`` and ``FITS``.
+    ``astropy.io.votable`` are ``TABLEDATA``, ``BINARY``, and ``FITS``.
+    In addition, ``astropy.io.votable`` also supports ``PARQUET`` serialization, which is
+    a candidate for addition to the VOTable specification.
 
     **References:** `1.1
     <http://www.ivoa.net/documents/VOTable/20040811/REC-VOTable-1.1-20040811.html#sec:data>`__,
@@ -1066,9 +1074,10 @@ class W49(VOTableSpecWarning):
 class W50(VOTableSpecWarning):
     """
     Invalid unit string as defined in the `Units in the VO, Version 1.0
-    <https://www.ivoa.net/documents/VOUnits>`_ (VOTable version >= 1.4)
-    or `Standards for Astronomical Catalogues, Version 2.0
-    <https://cdsarc.cds.unistra.fr/doc/catstd-3.2.htx>`_ (version < 1.4).
+    <https://www.ivoa.net/documents/VOUnits/20140523/index.html>`_
+    (VOTable version >= 1.4) or `Standards for Astronomical Catalogues,
+    Version 2.0 <https://cdsarc.cds.unistra.fr/doc/catstd-3.2.htx>`_
+    (version < 1.4).
 
     Consider passing an explicit ``unit_format`` parameter if the units
     in this file conform to another specification.
@@ -1135,6 +1144,33 @@ class W55(VOTableSpecWarning):
     default_args = ("", "")
 
 
+class W56(VOTableSpecWarning):
+    """
+    The column fields as defined using ``FIELD`` elements do not match
+    those in the headers of the embedded PARQUET file.  If ``verify`` is not
+    ``'exception'``, the embedded PARQUET file will take precedence.
+    """
+
+    message_template = (
+        "The fields defined in the VOTable do not match those in the "
+        "embedded PARQUET file"
+    )
+
+
+class W57(VOTableSpecWarning):
+    """
+    The ``refposition`` attribute on the ``COOSYS`` element is only allowed on
+    VOTABLE versions 1.5 and greater.
+    favor of a reference to the Space-Time Coordinate (STC) data
+    model (see `utype
+    <http://www.ivoa.net/documents/VOTable/20091130/REC-VOTable-1.2.html#sec:utype>`__
+    and the IVOA note `referencing STC in VOTable
+    <http://ivoa.net/Documents/latest/VOTableSTC.html>`__.
+    """
+
+    message_template = "refposition only allowed on VOTABLE v1.5 and greater"
+
+
 class E01(VOWarning, ValueError):
     """Invalid size specifier for a field.
 
@@ -1524,6 +1560,14 @@ class E25(VOTableSpecWarning):
     message_template = "No FIELDs are defined; DATA section will be ignored."
 
 
+class E26(VOTableSpecError):
+    """
+    The mapping block can only be set in a type=meta RESOURCE.
+    """
+
+    message_template = "Mapping block can not be set in a RESOURCE with type=result"
+
+
 def _get_warning_and_exception_classes(prefix):
     classes = []
     for key, val in globals().items():
@@ -1565,5 +1609,5 @@ def _build_doc_string():
 if __doc__ is not None:
     __doc__ = __doc__.format(**_build_doc_string())
 
-__all__.extend([x[0] for x in _get_warning_and_exception_classes("W")])
-__all__.extend([x[0] for x in _get_warning_and_exception_classes("E")])
+__all__ += [x[0] for x in _get_warning_and_exception_classes("W")]
+__all__ += [x[0] for x in _get_warning_and_exception_classes("E")]
