@@ -6,6 +6,7 @@ import pytest
 
 from astropy.io.fits import BinTableHDU, HDUList, Header, PrimaryHDU
 from astropy.timeseries.io.kepler import kepler_fits_reader
+from astropy.units import UnitsWarning
 from astropy.utils.data import get_pkg_data_filename
 
 
@@ -104,8 +105,51 @@ def test_tess_astropy():
     filename = get_pkg_data_filename(
         "timeseries/hlsp_tess-data-alerts_tess_phot_00025155310-s01_tess_v1_lc.fits"
     )
-    with pytest.warns(UserWarning, match="Ignoring 815 rows with NaN times"):
+    with pytest.warns((UserWarning, UnitsWarning)) as record:
         timeseries = kepler_fits_reader(filename)
+
+    # we might hit some warnings more than once, but the exact sequence probably
+    # does not matter too much, so we'll just try to match the *set* of unique warnings
+    unique_warnings = {(wm.category, wm.message.args[0]) for wm in record}
+    expected = {
+        (UserWarning, "Ignoring 815 rows with NaN times"),
+        (
+            UnitsWarning,
+            (
+                "'BJD - 2457000, days' did not parse as fits unit: "
+                "At col 0, Unit 'BJD' not supported by the FITS standard.  "
+                "If this is meant to be a custom unit, define it with 'u.def_unit'. "
+                "To have it recognized inside a file reader or other code, "
+                "enable it with 'u.add_enabled_units'. "
+                "For details, see https://docs.astropy.org/en/latest/units/combining_and_defining.html"
+            ),
+        ),
+        (
+            UnitsWarning,
+            (
+                "'pixels' did not parse as fits unit: "
+                "At col 0, Unit 'pixels' not supported by the FITS standard. "
+                "Did you mean pixel? If this is meant to be a custom unit, "
+                "define it with 'u.def_unit'. To have it recognized inside a file "
+                "reader or other code, enable it with 'u.add_enabled_units'. "
+                "For details, see https://docs.astropy.org/en/latest/units/combining_and_defining.html"
+            ),
+        ),
+        (
+            UnitsWarning,
+            (
+                "'e-/s' did not parse as fits unit: "
+                "At col 0, Unit 'e' not supported by the FITS standard.  "
+                "If this is meant to be a custom unit, define it with 'u.def_unit'. "
+                "To have it recognized inside a file reader or other code, "
+                "enable it with 'u.add_enabled_units'. "
+                "For details, see https://docs.astropy.org/en/latest/units/combining_and_defining.html"
+            ),
+        ),
+    }
+    assert (
+        unique_warnings == expected
+    ), f"Got some unexpected warnings\n{unique_warnings - expected}"
     assert timeseries["time"].format == "isot"
     assert timeseries["time"].scale == "tdb"
     assert timeseries["sap_flux"].unit.to_string() == "electron / s"
