@@ -5,7 +5,8 @@
 from erfa import dt_eraASTROM, dt_eraLDBODY, dt_pv
 from erfa import ufunc as erfa_ufunc
 
-from astropy.units.core import UnitsError, UnitTypeError, dimensionless_unscaled
+from astropy.units.core import dimensionless_unscaled
+from astropy.units.errors import UnitsError, UnitTypeError
 from astropy.units.structured import StructuredUnit
 
 from . import UFUNC_HELPERS
@@ -339,9 +340,9 @@ def helper_apio(
         get_converter(unit_phi, radian),
         get_converter(unit_hm, m),
         get_converter(unit_xp, radian),
-        get_converter(unit_xp, radian),
-        get_converter(unit_xp, radian),
-        get_converter(unit_xp, radian),
+        get_converter(unit_yp, radian),
+        get_converter(unit_refa, radian),
+        get_converter(unit_refb, radian),
     ], astrom_unit()
 
 
