@@ -9,13 +9,13 @@ from matplotlib.artist import Artist
 from matplotlib.axes import Axes, subplot_class_factory
 from matplotlib.transforms import Affine2D, Bbox, Transform
 
-import astropy.units as u
 from astropy.coordinates import BaseCoordinateFrame, SkyCoord
 from astropy.utils import minversion
 from astropy.utils.compat.optional_deps import HAS_PIL
 from astropy.wcs import WCS
 from astropy.wcs.wcsapi import BaseHighLevelWCS, BaseLowLevelWCS
 
+from ._auto import auto_assign_coord_positions
 from .coordinates_map import CoordinatesMap
 from .frame import RectangularFrame, RectangularFrame1D
 from .transforms import CoordinateTransform
@@ -39,9 +39,10 @@ class _WCSAxesArtist(Artist):
     ersatz ticks, labels, and gridlines by explicitly calling the functions
     ``CoordinateHelper._draw_ticks``, ``CoordinateHelper._draw_grid``, etc.
     This hack would not be necessary if ``WCSAxes`` drew ticks, tick labels,
-    and gridlines in the standary way."""
+    and gridlines in the standary way.
+    """
 
-    def draw(self, renderer, *args, **kwargs):
+    def draw(self, renderer):
         self.axes.draw_wcsaxes(renderer)
 
 
@@ -100,6 +101,11 @@ class WCSAxes(Axes):
         The class for the frame, which should be a subclass of
         :class:`~astropy.visualization.wcsaxes.frame.BaseFrame`. The default is to use a
         :class:`~astropy.visualization.wcsaxes.frame.RectangularFrame`
+
+    Attributes
+    ----------
+    coords : :class:`~astropy.visualization.wcsaxes.CoordinatesMap`
+        Container for coordinate information.
     """
 
     def __init__(
@@ -128,7 +134,7 @@ class WCSAxes(Axes):
         else:
             self.frame_class = RectangularFrame
 
-        if not (transData is None):
+        if transData is not None:
             # User wants to override the transform for the final
             # data->pixel mapping
             self.transData = transData
@@ -206,7 +212,6 @@ class WCSAxes(Axes):
 
         All arguments are passed to :meth:`~matplotlib.axes.Axes.imshow`.
         """
-
         origin = kwargs.pop("origin", "lower")
 
         # plt.imshow passes origin as None, which we should default to lower.
@@ -240,7 +245,6 @@ class WCSAxes(Axes):
         positional and keyword arguments are the same as for
         :meth:`~matplotlib.axes.Axes.contour`.
         """
-
         # In Matplotlib, when calling contour() with a transform, each
         # individual path in the contour map is transformed separately. However,
         # this is much too slow for us since each call to the transforms results
@@ -272,7 +276,6 @@ class WCSAxes(Axes):
         positional and keyword arguments are the same as for
         :meth:`~matplotlib.axes.Axes.contourf`.
         """
-
         # See notes for contour above.
 
         transform = kwargs.pop("transform", None)
@@ -291,7 +294,7 @@ class WCSAxes(Axes):
     def _transform_plot_args(self, *args, **kwargs):
         """
         Apply transformations to arguments to ``plot_coord`` and
-        ``scatter_coord``
+        ``scatter_coord``.
         """
         if isinstance(args[0], (SkyCoord, BaseCoordinateFrame)):
             # Extract the frame from the first argument.
@@ -306,9 +309,9 @@ class WCSAxes(Axes):
             plot_data = []
             for coord in self.coords:
                 if coord.coord_type == "longitude":
-                    plot_data.append(frame0.spherical.lon.to_value(u.deg))
+                    plot_data.append(frame0.spherical.lon.to_value(coord.coord_unit))
                 elif coord.coord_type == "latitude":
-                    plot_data.append(frame0.spherical.lat.to_value(u.deg))
+                    plot_data.append(frame0.spherical.lat.to_value(coord.coord_unit))
                 else:
                     raise NotImplementedError(
                         "Coordinates cannot be plotted with this "
@@ -352,11 +355,38 @@ class WCSAxes(Axes):
             This method is called from this function with all arguments passed to it.
 
         """
-
         args, kwargs = self._transform_plot_args(*args, **kwargs)
 
         return super().plot(*args, **kwargs)
 
+    def text_coord(self, *args, **kwargs):
+        """
+        Print a text string using `~astropy.coordinates.SkyCoord` or
+        `~astropy.coordinates.BaseCoordinateFrame` objects onto the axes.
+
+        The first argument to
+        :meth:`~astropy.visualization.wcsaxes.WCSAxes.text_coord` should be a
+        coordinate, which will then be converted to the first two parameters to
+        `matplotlib.axes.Axes.text`. All other arguments are the same as
+        `matplotlib.axes.Axes.text`. If not specified a ``transform`` keyword
+        argument will be created based on the coordinate.
+
+        Parameters
+        ----------
+        coordinate : `~astropy.coordinates.SkyCoord` or `~astropy.coordinates.BaseCoordinateFrame`
+            The coordinate object to plot on the axes. This is converted to the
+            first two arguments to `matplotlib.axes.Axes.text`.
+
+        See Also
+        --------
+        matplotlib.axes.Axes.text :
+            This method is called from this function with all arguments passed to it.
+
+        """
+        args, kwargs = self._transform_plot_args(*args, **kwargs)
+
+        return super().text(*args, **kwargs)
+
     def scatter_coord(self, *args, **kwargs):
         """
         Scatter `~astropy.coordinates.SkyCoord` or
@@ -379,7 +409,6 @@ class WCSAxes(Axes):
         --------
         matplotlib.axes.Axes.scatter : This method is called from this function with all arguments passed to it.
         """
-
         args, kwargs = self._transform_plot_args(*args, **kwargs)
 
         return super().scatter(*args, **kwargs)
@@ -388,7 +417,6 @@ class WCSAxes(Axes):
         """
         Reset the current Axes, to use a new WCS object.
         """
-
         # Here determine all the coordinate axes that should be shown.
         if wcs is None and transform is None:
             self.wcs = IDENTITY
@@ -463,6 +491,33 @@ class WCSAxes(Axes):
         if rcParams["axes.grid"]:
             self.grid()
 
+    def _update_tick_and_label_positions(self, keep_coord_range=False):
+        """
+        This method will update the tick positions and will then optionally
+        decide on which axes to show ticks/tick labels/axis labels on if in
+        automatic mode.
+
+        The ``keep_coord_range`` argument is used to indicate whether to keep
+        coords._coord_range at the end of the method or whether to clean it
+        up.
+        """
+        # Start off by updating the frame, pre-computing the coordinate range
+        # in the figure, and updating the tick positions.
+        for coords in self._all_coords:
+            coords.frame.update()
+            coords._coord_range = coords.get_coord_range()
+
+            for coord in coords:
+                coord._update_ticks()
+
+        # At this point, if any of the tick/ticklabel/axislabel positions are
+        # set to be automatic, we need to determine the optimal positions.
+        auto_assign_coord_positions(self)
+
+        if not keep_coord_range:
+            for coords in self._all_coords:
+                del coords._coord_range
+
     def draw_wcsaxes(self, renderer):
         if not self.axison:
             return
@@ -475,21 +530,24 @@ class WCSAxes(Axes):
 
         visible_ticks = []
 
+        self._update_tick_and_label_positions(keep_coord_range=True)
+
         for coords in self._all_coords:
-            # Draw grids
-            coords.frame.update()
             for coord in coords:
                 coord._draw_grid(renderer)
 
+            # Delete the computation to protect from accidental use of a stale range
+            del coords._coord_range
+
         for coords in self._all_coords:
             # Draw tick labels
             for coord in coords:
-                coord._draw_ticks(
-                    renderer,
-                    bboxes=self._bboxes,
-                    ticklabels_bbox=ticklabels_bbox[coord],
-                )
-                visible_ticks.extend(coord.ticklabels.get_visible_axes())
+                coord._draw_ticks(renderer, self._bboxes)
+
+                visible_ticks.extend(coord._ticklabels.get_visible_axes())
+                # Save ticklabel bboxes
+                ticklabels_bbox[coord] = coord._ticklabels._axis_bboxes
+                self._bboxes += coord._ticklabels._all_bboxes
 
         for coords in self._all_coords:
             # Draw axis labels
@@ -503,9 +561,8 @@ class WCSAxes(Axes):
 
         self.coords.frame.draw(renderer)
 
-    def draw(self, renderer, **kwargs):
+    def draw(self, renderer):
         """Draw the axes."""
-
         # Before we do any drawing, we need to remove any existing grid lines
         # drawn with contours, otherwise if we try and remove the contours
         # part way through drawing, we end up with the issue mentioned in
@@ -535,13 +592,14 @@ class WCSAxes(Axes):
         # We need to make sure that that frame path is up to date
         self.coords.frame._update_patch_path()
 
-        super().draw(renderer, **kwargs)
+        super().draw(renderer)
 
         self._drawn = True
 
     # Matplotlib internally sometimes calls set_xlabel(label=...).
     def set_xlabel(self, xlabel=None, labelpad=1, loc=None, **kwargs):
         """Set x-label."""
+        self._update_tick_and_label_positions()
         if xlabel is None:
             xlabel = kwargs.pop("label", None)
             if xlabel is None:
@@ -550,14 +608,15 @@ class WCSAxes(Axes):
                 )
         for coord in self.coords:
             if (
-                "b" in coord.axislabels.get_visible_axes()
-                or "h" in coord.axislabels.get_visible_axes()
+                "b" in coord._axislabels.get_visible_axes()
+                or "h" in coord._axislabels.get_visible_axes()
             ):
                 coord.set_axislabel(xlabel, minpad=labelpad, **kwargs)
                 break
 
     def set_ylabel(self, ylabel=None, labelpad=1, loc=None, **kwargs):
-        """Set y-label"""
+        """Set y-label."""
+        self._update_tick_and_label_positions()
         if ylabel is None:
             ylabel = kwargs.pop("label", None)
             if ylabel is None:
@@ -570,32 +629,51 @@ class WCSAxes(Axes):
 
         for coord in self.coords:
             if (
-                "l" in coord.axislabels.get_visible_axes()
-                or "c" in coord.axislabels.get_visible_axes()
+                "l" in coord._axislabels.get_visible_axes()
+                or "c" in coord._axislabels.get_visible_axes()
             ):
                 coord.set_axislabel(ylabel, minpad=labelpad, **kwargs)
                 break
 
     def get_xlabel(self):
+        self._update_tick_and_label_positions()
         for coord in self.coords:
             if (
-                "b" in coord.axislabels.get_visible_axes()
-                or "h" in coord.axislabels.get_visible_axes()
+                "b" in coord._axislabels.get_visible_axes()
+                or "h" in coord._axislabels.get_visible_axes()
             ):
                 return coord.get_axislabel()
 
     def get_ylabel(self):
+        self._update_tick_and_label_positions()
         if self.frame_class is RectangularFrame1D:
             return super().get_ylabel()
 
         for coord in self.coords:
             if (
-                "l" in coord.axislabels.get_visible_axes()
-                or "c" in coord.axislabels.get_visible_axes()
+                "l" in coord._axislabels.get_visible_axes()
+                or "c" in coord._axislabels.get_visible_axes()
             ):
                 return coord.get_axislabel()
 
     def get_coords_overlay(self, frame, coord_meta=None):
+        """Get coordinates overlay on given frame.
+
+        Parameters
+        ----------
+        frame : str, `~astropy.coordinates.BaseCoordinateFrame`
+            Frame to get overlay for. If a string must correspond to
+            one of the coordinate frames registered in the astropy
+            frame transform graph.
+
+        coord_meta : dict
+            Metadata for the coordinates overlay.
+
+        Returns
+        -------
+        overlay : `~astropy.visualization.wcsaxes.CoordinatesMap`
+            Coordinates overlay.
+        """
         # Here we can't use get_transform because that deals with
         # pixel-to-pixel transformations when passing a WCS object.
         if isinstance(frame, WCS):
@@ -631,7 +709,7 @@ class WCSAxes(Axes):
         """
         Return a transform from the specified frame to display coordinates.
 
-        This does not include the transData transformation
+        This includes the transData transformation
 
         Parameters
         ----------
@@ -662,9 +740,8 @@ class WCSAxes(Axes):
 
     def _get_transform_no_transdata(self, frame):
         """
-        Return a transform from data to the specified frame
+        Return a transform from data to the specified frame.
         """
-
         if isinstance(frame, (BaseLowLevelWCS, BaseHighLevelWCS)):
             if isinstance(frame, BaseHighLevelWCS):
                 frame = frame.low_level_wcs
@@ -744,7 +821,6 @@ class WCSAxes(Axes):
         which : str
             Currently only ``'major'`` is supported.
         """
-
         if not hasattr(self, "coords"):
             return
 
@@ -821,7 +897,6 @@ class WCSAxes(Axes):
             The style of the grid lines (accepts any valid Matplotlib line
             style).
         """
-
         if not hasattr(self, "coords"):
             # Axes haven't been fully initialized yet, so just ignore, as
             # Axes.__init__ calls this method
@@ -843,8 +918,10 @@ class WCSAxes(Axes):
         elif axis in ("x", "y") and self.frame_class is RectangularFrame:
             spine = "b" if axis == "x" else "l"
 
+            self._update_tick_and_label_positions()
+
             for coord in self.coords:
-                if spine in coord.axislabels.get_visible_axes():
+                if spine in coord._axislabels.get_visible_axes():
                     coord.tick_params(**kwargs)
 
 
@@ -855,7 +932,5 @@ class WCSAxes(Axes):
 
 class WCSAxesSubplot(subplot_class_factory(WCSAxes)):
     """
-    A subclass class for WCSAxes
+    A subclass class for WCSAxes.
     """
-
-    pass
