@@ -4,7 +4,7 @@
 import numpy as np
 from matplotlib.lines import Path
 
-from astropy.coordinates.angle_utilities import angular_separation
+from astropy.coordinates import angular_separation
 
 # Tolerance for WCS round-tripping, relative to the scale size
 ROUND_TRIP_RTOL = 1.0
@@ -28,7 +28,6 @@ def get_lon_lat_path(lon_lat, pixel, lon_lat_check):
         The world coordinates derived from converting from ``pixel``, which is
         used to ensure round-tripping.
     """
-
     # In some spherical projections, some parts of the curve are 'behind' or
     # 'in front of' the plane of the image, so we find those by reversing the
     # transformation and finding points where the result is not consistent.
@@ -93,7 +92,7 @@ def get_lon_lat_path(lon_lat, pixel, lon_lat_check):
 
 def get_gridline_path(world, pixel):
     """
-    Draw a grid line
+    Draw a grid line.
 
     Parameters
     ----------
@@ -103,7 +102,6 @@ def get_gridline_path(world, pixel):
     pixel : ndarray
         The pixel coordinates corresponding to ``lon_lat``
     """
-
     # Mask values with invalid pixel positions
     mask = np.isnan(pixel[:, 0]) | np.isnan(pixel[:, 1])
 
