@@ -56,15 +56,15 @@ from astropy.time import Time
 #
 # Get the coordinates of M33:
 
-m33 = SkyCoord.from_name('M33')
+m33 = SkyCoord.from_name("M33")
 
 ##############################################################################
 # Use `astropy.coordinates.EarthLocation` to provide the location of Bear
 # Mountain and set the time to 11pm EDT on 2012 July 12:
 
-bear_mountain = EarthLocation(lat=41.3*u.deg, lon=-74*u.deg, height=390*u.m)
-utcoffset = -4*u.hour  # Eastern Daylight Time
-time = Time('2012-7-12 23:00:00') - utcoffset
+bear_mountain = EarthLocation(lat=41.3 * u.deg, lon=-74 * u.deg, height=390 * u.m)
+utcoffset = -4 * u.hour  # Eastern Daylight Time
+time = Time("2012-7-12 23:00:00") - utcoffset
 
 ##############################################################################
 # `astropy.coordinates.EarthLocation.get_site_names` and
@@ -74,7 +74,7 @@ time = Time('2012-7-12 23:00:00') - utcoffset
 # Use `astropy.coordinates` to find the Alt, Az coordinates of M33 at as
 # observed from Bear Mountain at 11pm on 2012 July 12.
 
-m33altaz = m33.transform_to(AltAz(obstime=time,location=bear_mountain))
+m33altaz = m33.transform_to(AltAz(obstime=time, location=bear_mountain))
 print(f"M33's Altitude = {m33altaz.alt:.2}")
 
 ##############################################################################
@@ -85,10 +85,9 @@ print(f"M33's Altitude = {m33altaz.alt:.2}")
 # Find the alt,az coordinates of M33 at 100 times evenly spaced between 10pm
 # and 7am EDT:
 
-midnight = Time('2012-7-13 00:00:00') - utcoffset
-delta_midnight = np.linspace(-2, 10, 100)*u.hour
-frame_July13night = AltAz(obstime=midnight+delta_midnight,
-                          location=bear_mountain)
+midnight = Time("2012-7-13 00:00:00") - utcoffset
+delta_midnight = np.linspace(-2, 10, 100) * u.hour
+frame_July13night = AltAz(obstime=midnight + delta_midnight, location=bear_mountain)
 m33altazs_July13night = m33.transform_to(frame_July13night)
 
 ##############################################################################
@@ -102,8 +101,8 @@ m33airmasss_July13night = m33altazs_July13night.secz
 plt.plot(delta_midnight, m33airmasss_July13night)
 plt.xlim(-2, 10)
 plt.ylim(1, 4)
-plt.xlabel('Hours from EDT Midnight')
-plt.ylabel('Airmass [Sec(z)]')
+plt.xlabel("Hours from EDT Midnight")
+plt.ylabel("Airmass [Sec(z)]")
 plt.show()
 
 ##############################################################################
@@ -112,7 +111,7 @@ plt.show()
 
 from astropy.coordinates import get_sun
 
-delta_midnight = np.linspace(-12, 12, 1000)*u.hour
+delta_midnight = np.linspace(-12, 12, 1000) * u.hour
 times_July12_to_13 = midnight + delta_midnight
 frame_July12_to_13 = AltAz(obstime=times_July12_to_13, location=bear_mountain)
 sunaltazs_July12_to_13 = get_sun(times_July12_to_13).transform_to(frame_July12_to_13)
@@ -137,20 +136,40 @@ m33altazs_July12_to_13 = m33.transform_to(frame_July12_to_13)
 # Make a beautiful figure illustrating nighttime and the altitudes of M33 and
 # the Sun over that time:
 
-plt.plot(delta_midnight, sunaltazs_July12_to_13.alt, color='r', label='Sun')
-plt.plot(delta_midnight, moonaltazs_July12_to_13.alt, color=[0.75]*3, ls='--', label='Moon')
-plt.scatter(delta_midnight, m33altazs_July12_to_13.alt,
-            c=m33altazs_July12_to_13.az.value, label='M33', lw=0, s=8,
-            cmap='viridis')
-plt.fill_between(delta_midnight, 0*u.deg, 90*u.deg,
-                 sunaltazs_July12_to_13.alt < -0*u.deg, color='0.5', zorder=0)
-plt.fill_between(delta_midnight, 0*u.deg, 90*u.deg,
-                 sunaltazs_July12_to_13.alt < -18*u.deg, color='k', zorder=0)
-plt.colorbar().set_label('Azimuth [deg]')
-plt.legend(loc='upper left')
-plt.xlim(-12*u.hour, 12*u.hour)
-plt.xticks((np.arange(13)*2-12)*u.hour)
-plt.ylim(0*u.deg, 90*u.deg)
-plt.xlabel('Hours from EDT Midnight')
-plt.ylabel('Altitude [deg]')
+plt.plot(delta_midnight, sunaltazs_July12_to_13.alt, color="r", label="Sun")
+plt.plot(
+    delta_midnight, moonaltazs_July12_to_13.alt, color=[0.75] * 3, ls="--", label="Moon"
+)
+plt.scatter(
+    delta_midnight,
+    m33altazs_July12_to_13.alt,
+    c=m33altazs_July12_to_13.az.value,
+    label="M33",
+    lw=0,
+    s=8,
+    cmap="viridis",
+)
+plt.fill_between(
+    delta_midnight,
+    0 * u.deg,
+    90 * u.deg,
+    sunaltazs_July12_to_13.alt < -0 * u.deg,
+    color="0.5",
+    zorder=0,
+)
+plt.fill_between(
+    delta_midnight,
+    0 * u.deg,
+    90 * u.deg,
+    sunaltazs_July12_to_13.alt < -18 * u.deg,
+    color="k",
+    zorder=0,
+)
+plt.colorbar().set_label("Azimuth [deg]")
+plt.legend(loc="upper left")
+plt.xlim(-12 * u.hour, 12 * u.hour)
+plt.xticks((np.arange(13) * 2 - 12) * u.hour)
+plt.ylim(0 * u.deg, 90 * u.deg)
+plt.xlabel("Hours from EDT Midnight")
+plt.ylabel("Altitude [deg]")
 plt.show()
